/* $Id: certsec.c,v 1.6 2016/03/23 15:17:50 tm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 *
 * Encrypt file with certificate security.
 *
 */

#include <stdio.h>

#include "ploplib.h"

int
main(int argc, char **argv)
{
    PLOP *plop;

    /* This is where input files live. Adjust as necessary. */
    const char *searchpath = "../data";

    /* Recipient certificates and permissions */
    typedef struct {const char *cert; const char *perm;} recipientdata;

    const recipientdata recipients[] = {
        /* no permission restrictions */
        { "../data/demo_recipient_1.pem", "" },

        /* form filling and signing permissions granted, but not others */
        { "../data/demo_recipient_2.pem", "nomodify nocopy noannots noassemble" },
        { "../data/demo_recipient_3.pem", "nomodify nocopy noannots noassemble" },
    };
#define RECIPIENTCOUNT (sizeof(recipients)/sizeof(recipients[0]))

    if (argc < 3 )
    {
        printf("usage: certsec <infile> <outfile>\n");
        return 2;
    }

    /* create a new PLOP object */
    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(2);
    }

    PLOP_TRY(plop) {
	int doc;
	size_t r;
	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	/* open unprotected input file */
	if ((doc = PLOP_open_document(plop, argv[1], 0, "")) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	/* add recipients */
	for (r=0; r<RECIPIENTCOUNT; r++)
	{
	    sprintf(optlist, "certificate={filename={%s}} permissions={%s}",
	            recipients[r].cert, recipients[r].perm);
            if (PLOP_add_recipient(plop, optlist) == -1)
            {
                /* emit warning and continue */
                printf("Warning: %s\n", PLOP_get_errmsg(plop));
            }
	}

	/* create protected output file */
	sprintf(optlist, "input=%d", doc);
	if (PLOP_create_document(plop, argv[2], 0, optlist) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	/* close input and output files */
        PLOP_close_document(plop, doc, "");
    }

    PLOP_CATCH(plop) {
	printf("PLOP exception occurred in certsec sample:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(2);
    }

    /* delete the PLOP object */
    PLOP_delete(plop);

    return(0);
}
