/* $Id: HelloSignServlet.java,v 1.7 2016/03/24 06:46:13 rp Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * hellosign sample: generate a document in memory with PDFlib, and sign it
 * with PLOP
 */

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

import com.pdflib.PDFlibException;
import com.pdflib.PLOPException;
import com.pdflib.pdflib;
import com.pdflib.plop;

public class HelloSignServlet extends HttpServlet {
    private static final long serialVersionUID = 1177865317978810551L;

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException {
        pdflib pdf = null;
        plop plop = null;

        try {
            /*
             * Set up searchpath for the required data.
             */
            final ServletContext context = getServletContext();
            final String searchpath = context.getRealPath("/WEB-INF/data");

            /*
             * Catch problems that application data cannot be accessed.
             */
            if (searchpath == null) {
                throw new ServletException(
                        "Error: Unable to access application data (WAR file not unpacked?)");
            }

            /* Basic signature options */
            String sign_opts =
                "ltv=try " +         /* use ltv=full to ensure LTV status */
                /* for use with PKCS11
                "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
                */
                "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} ";

            /* ----------- Step 1: generate the document in memory ---------- */

            pdf = new pdflib();

            pdf.begin_document("", "");

            pdf.set_info("Creator", "HelloSignServlet.java");
            pdf.set_info("Author", "Rainer Pl\u00f6ckl");
            pdf.set_info("Title", "Hello world/signed (Java/Servlet)");

            final int font = pdf.load_font("Helvetica-Bold", "winansi", "");

            pdf.begin_page_ext(595, 842, "");
            pdf.setfont(font, 24);
            pdf.set_text_pos(50, 700);
            pdf.show("Hello world/signed!");
            pdf.continue_text("(says Java/Servlet)");
            pdf.end_page_ext("");

            pdf.end_document("");
            
            final byte[] contents = pdf.get_buffer();

            /* --------------- Step 2: sign the document -------------- */

            plop = new plop();

                /* parameters for the input document */
            String optlist;
            String fieldoptlist = "";
            
            int doc;
            int visdoc =-1;
            boolean visualize = false;
            String visfile = "signing_man_pdfa1a.pdf";

            plop = new plop();

            /* open unprotected input file without any password */
            optlist = "searchpath {" + searchpath + "} ";
            plop.set_option(optlist);
            /* Create a virtual PVF file from the document data in memory */
            plop.create_pvf("/pvf/hello.pdf", contents, "");

            /* open unsigned input document from PVF file */
            if ((doc = plop.open_document("/pvf/hello.pdf", "")) == -1) {
              throw new Exception("Error: " + plop.get_errmsg());
            }
           
            optlist = sign_opts;

            if (visualize) {
              if ((visdoc = plop.open_document(visfile, "")) == -1) {
                  throw new Exception("Error: " + plop.get_apiname()
                        + ": " + plop.get_errmsg());
              }
              fieldoptlist = "field={visdoc=" + visdoc + " rect={50 50 adapt adapt}} ";
              optlist = optlist + fieldoptlist;
            }

            if (plop.prepare_signature(optlist) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                                              + ": " + plop.get_errmsg());
            }

            if (plop.create_document("", "input=" + doc) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                                              + ": " + plop.get_errmsg());
            }
            
            final byte[] buf = plop.get_buffer();

            /* close input and output files */
            plop.close_document(doc, "");
            if (visualize) {
                plop.close_document(visdoc, "");
            }

            response.setContentType("application/pdf");
            response.setContentLength(buf.length);

            final ServletOutputStream out = response.getOutputStream();
            out.write(buf);
            out.close();

            /* delete PVF file containing input document */
            plop.delete_pvf("/pvf/hello.pdf");
        }
        catch (PLOPException e) {
            throw new ServletException("PLOP exception occurred in "
                    + getClass().getName(), e);
        }
        catch (PDFlibException e) {
            throw new ServletException("PDFlib exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            /* delete the PLOP object */
            if (plop != null)
                plop.delete();

            /* delete the PDFlib object */
            if (pdf != null) {
                pdf.delete();
            }
        }
    }
}
