<?php
    /* $Id: insertxmp.php,v 1.4 2014/09/04 08:44:17 rp Exp $
     * PDFlib PLOP: PDF Linearization, Optimization, Protection
     * mini sample for inserting XMP metadata in PDF documents
     */
try{
    /* Directory where files will be searched */
    $searchpath = "../data ../../data";

    /* input document */
    $in_filename = "PLOP-datasheet.pdf";

    /* output document */
    $out_filename = "";

    /* Option list for the generated output document.
     * Some alternatives for the option list using the supplied XMP samples:
     *
     * Insert custom metadata for the GWG (Ghent Workgroup)
     * Ad Ticket scheme (see http://www.gwg.org/Jobtickets.phtml):
     * "metadata={filename=gwg_ad_ticket.xmp validate=xmp2004}"
     *
     * Insert custom metadata based on Acme's enterprise schema:
     * "metadata={filename=acme.xmp}"
     *
     * Insert a custom "engineering" schema for scanned documents:
     * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
     * and requires XMP input which conforms to PDF/A-1:
     * "metadata={filename=engineering.xmp}"
     *
     * Insert a PDF/A extension schema after validating the XMP for PDF/A-1.
     * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
     * and requires XMP input which conforms to PDF/A-1:
     * "metadata={filename=machine_pdfa1.xmp validate=pdfa1}"
     *
     */
    $base_opts = "metadata={filename=gwg_ad_ticket.xmp}";

    /* create a new PLOP object */
    $plop = new PLOP();

    $optlist = sprintf("searchpath={%s}", $searchpath);
    $plop->set_option($optlist);

    /* open input file */
    if (!($doc = $plop->open_document($in_filename, ""))) {
	die("Error: " . $plop->get_errmsg());
    }

    /* create the output file and add XMP metadata */
    if (!($plop->create_document($out_filename, $base_opts . " input=" . $doc))) {
	die("Error: " . $plop->get_errmsg());
    }

    $buf = $plop->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=insertxmp.pdf");
    print $buf;

    /* close input files */
    $plop->close_document($doc, "");
}
catch (PLOPException $e) {
    die("PLOP exception occurred in insertxmp:" .
        "[" . $e->get_errnum() . "] " . $e->get_apiname() . ": " .
        $e->get_errmsg() . "\n");
    }
catch (Exception $e) {
    die($e);
}

$plop = 0;
?>
